searchNodes=[{"ref":"Pipehammer.html","title":"Pipehammer","type":"module","doc":"Pipehammer, the simplest pipeline management. You may now declare regular functions, and Pipehammer will write boilerplate for you. Example: defpipe safe_div(x, 0) do {:error, :division_by_zero} end defpipe safe_div(x, y) do {:ok, x / y} end This will define a function having this type signature: @spec safe_div(any | {:ok, any} | {:error, any} :: {:ok, any} | {:error, :division_by_zero} | {:error, any } 12 |&gt; safe_div(6) # =&gt; {:ok, 2.0} 12 |&gt; safe_div(0) # =&gt; {:error, :division_by_zero} {:ok, 12} |&gt; safe_div(6) # =&gt; {:ok, 2.0} {:error, :not_a_number} |&gt; safe_div(6) # =&gt; {:error, :not_a_number} This has immediate application in building pipelines: defp issue_referral_voucher(account_id, referrer_reference) do find_account(account_id) |&gt; set_account_referrer(referrer_reference) |&gt; issue_referral_voucher() |&gt; create_activity() |&gt; publish_event() |&gt; handle_error(fn e -&gt; {:error, &quot;bad error&quot;} end) end You can still use with macro if you wish: defp issue_referral_voucher(account_id, referrer_reference) do with {:ok, acc} &lt;- find_account(account_id), {:ok, acc} &lt;- set_account_referrer(acc, referrer_reference), {:ok, vou} &lt;- issue_referral_voucher(acc), {:ok, act} &lt;- create_activity(vou) do publish_event(act) else {:error, e} -&gt; {:error, &quot;bad error&quot;} end end You can still compose the functions as usual: set_account_referrer(find_account(account_id), referrer_reference) Error handling Pipe functions are expected to return {:error, e} tuples in error cases. If additional information is required, consider wrapping it into a tuple: {:error, {:foo, :bar}} If pipe function returns anything other than {:ok, any} | {:error, any} the behaviour of Pipehammer is undefined (means, the pipeline will crash in production)) What boilerplate to expect: defpipe partial(x, y) when is_integer(x) do {:ok, x + y} end Will create these functions, in order: def partial({:error, e}, _) do {:error, e} end def partial({:ok, x}, y) when is_integer(x) do partial(x, y) end def partial(x, y) when is_integer(x) do {:ok, x + y} end Note that error case intentionally does not have guards, as it does not care what you pass into the function, as long as 1st argument was an error. This logic is essential to chaining the steps. Pipehammer is smart to eliminate duplicate error declarations: defpipe foo(x, 0) do {:ok, x} end defpipe foo(x, 1) do {:ok, x} end This will still write one error clause: def foo({:error, e}, _) do {:error, e} end"},{"ref":"Pipehammer.html#<|>/2","title":"Pipehammer.<|>/2","type":"macro","doc":"Alternative piping: it first step fails, try another one."},{"ref":"Pipehammer.html#defpipe/2","title":"Pipehammer.defpipe/2","type":"macro","doc":"Declares function clauses that will handle nil, {:ok, _}, {:error, _} in addition to its main clause."},{"ref":"Pipehammer.html#handle_error/2","title":"Pipehammer.handle_error/2","type":"function","doc":"Handles errors, but leaves ok tuples intact"},{"ref":"Pipehammer.Error.html","title":"Pipehammer.Error","type":"module","doc":"Opt-in macros for simpler error construction and matching. It works with Pipehammer error handler, too."},{"ref":"Pipehammer.Error.html#error/1","title":"Pipehammer.Error.error/1","type":"macro","doc":""},{"ref":"Pipehammer.Error.html#error/2","title":"Pipehammer.Error.error/2","type":"macro","doc":""},{"ref":"Pipehammer.Error.html#error/3","title":"Pipehammer.Error.error/3","type":"macro","doc":""},{"ref":"Pipehammer.Error.html#error/4","title":"Pipehammer.Error.error/4","type":"macro","doc":""},{"ref":"Pipehammer.Error.html#error/5","title":"Pipehammer.Error.error/5","type":"macro","doc":""},{"ref":"Pipehammer.Error.html#t:error/1","title":"Pipehammer.Error.error/1","type":"type","doc":""},{"ref":"Pipehammer.Error.html#t:error/2","title":"Pipehammer.Error.error/2","type":"type","doc":""},{"ref":"Pipehammer.Error.html#t:error/3","title":"Pipehammer.Error.error/3","type":"type","doc":""},{"ref":"Pipehammer.Error.html#t:error/4","title":"Pipehammer.Error.error/4","type":"type","doc":""},{"ref":"Pipehammer.Error.html#t:error/5","title":"Pipehammer.Error.error/5","type":"type","doc":""},{"ref":"Pipehammer.Examples.html","title":"Pipehammer.Examples","type":"module","doc":"Examples for Pipehammer."},{"ref":"Pipehammer.Examples.html#add/2","title":"Pipehammer.Examples.add/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#add/3","title":"Pipehammer.Examples.add/3","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#bar/2","title":"Pipehammer.Examples.bar/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#baz/2","title":"Pipehammer.Examples.baz/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#destructure_1/2","title":"Pipehammer.Examples.destructure_1/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#destructure_2/2","title":"Pipehammer.Examples.destructure_2/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#exotic_errors/2","title":"Pipehammer.Examples.exotic_errors/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#foo/2","title":"Pipehammer.Examples.foo/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#ignore_x/2","title":"Pipehammer.Examples.ignore_x/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#ignore_y/2","title":"Pipehammer.Examples.ignore_y/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#mul/2","title":"Pipehammer.Examples.mul/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#partial/1","title":"Pipehammer.Examples.partial/1","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#partial2/2","title":"Pipehammer.Examples.partial2/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#rec_length/1","title":"Pipehammer.Examples.rec_length/1","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#safe_div/2","title":"Pipehammer.Examples.safe_div/2","type":"function","doc":""},{"ref":"Pipehammer.Examples.html#strange_minus/2","title":"Pipehammer.Examples.strange_minus/2","type":"function","doc":""},{"ref":"Pipehammer.Maybe.html","title":"Pipehammer.Maybe","type":"module","doc":"Wrappers for turning nil and structs to result tuples."},{"ref":"Pipehammer.Maybe.html#assert_struct/3","title":"Pipehammer.Maybe.assert_struct/3","type":"function","doc":""},{"ref":"Pipehammer.Maybe.html#from_maybe/2","title":"Pipehammer.Maybe.from_maybe/2","type":"function","doc":""},{"ref":"readme.html","title":"Pipehammer","type":"extras","doc":"Pipehammer Anyone can use a (pipe)hammer."},{"ref":"readme.html#installation","title":"Pipehammer - Installation","type":"extras","doc":"If available in Hex, the package can be installed by adding pipehammer to your list of dependencies in mix.exs: def deps do [ {:pipehammer, &quot;~&gt; 0.1.0&quot;} ] end Documentation can be generated with ExDoc and published on HexDocs. Once published, the docs can be found at https://hexdocs.pm/pipehammer."}]